package com.dsic.gj_erp.service.zyjh;

import cn.hutool.core.date.DateTime;
import cn.hutool.core.date.DateUtil;
import cn.hutool.core.util.ObjUtil;
import cn.hutool.core.util.StrUtil;
import com.baomidou.mybatisplus.core.conditions.query.QueryWrapper;
import com.baomidou.mybatisplus.core.conditions.update.LambdaUpdateWrapper;
import com.baomidou.mybatisplus.core.toolkit.Wrappers;
import com.baomidou.mybatisplus.extension.service.impl.ServiceImpl;
import com.dsic.gj_erp.bean.jcsj.DmBzry;
import com.dsic.gj_erp.bean.jcsj.DmCbxxp;
import com.dsic.gj_erp.bean.jhgk.DmYdjh;
import com.dsic.gj_erp.controller.zyjh.dto.PgWithPcDto;
import com.dsic.gj_erp.controller.zyjh.dto.ShangLiao;
import com.dsic.gj_erp.mapper.jhgk.DmYdjhMapper;
import com.dsic.gj_erp.service.jcsj.DmBzryService;
import com.dsic.gj_erp.service.jcsj.DmCbxxpService;
import com.google.common.collect.ImmutableMap;
import lombok.AllArgsConstructor;
import lombok.NonNull;
import org.springframework.cache.annotation.Cacheable;
import org.springframework.data.redis.core.RedisTemplate;
import org.springframework.stereotype.Service;

import java.util.HashMap;
import java.util.List;
import java.util.Map;
import java.util.stream.Collectors;

@Service
@AllArgsConstructor
public class ZyjhService extends ServiceImpl<DmYdjhMapper,DmYdjh> {

    private final DmBzryService bzryService;
    private final DmCbxxpService dmCbxxpService;
    private final RedisTemplate<String,Object> redisTemplate;

    public List<DmYdjh> getYdjhByZtOrForeman(Integer zt,String foreman){
        QueryWrapper<DmYdjh> wrapper = new QueryWrapper<DmYdjh>()
                .ge("zt",11).ge(ObjUtil.isNotEmpty(zt),"zt",zt);

        switch (zt){
            case 31:
                wrapper.eq("pwpgry",foreman).isNull("pwfkry");
                break;
            case 51:
                wrapper.eq("hxpgry",foreman).isNull("hxfkry");
                break;
            case 61:
                wrapper.eq("qgpgry",foreman).isNull("qgfkry");
                break;
        }
        wrapper.orderByDesc("dc_ch","dc_pl,dc_fd,tzbh");
        return list(wrapper);
    }

    public List<Map<String,Object>> getYdjhByZt(Integer zt,String wgzt,String wclzt,String zyq){
        DateTime date = DateUtil.date();
        String from = DateUtil.beginOfMonth(date).toString("yyyy/MM/dd");
        String to = DateUtil.endOfMonth(date).toString("yyyy/MM/dd");
        return baseMapper.getYdjhByZt(zyq,zt,wgzt,wclzt,from,to);
    }


    public void pgWithForeman(PgWithPcDto dto){
        LambdaUpdateWrapper<DmYdjh> wrapper = new LambdaUpdateWrapper<DmYdjh>()
                .eq(DmYdjh::getDcCh,dto.getDcCh()).eq(DmYdjh::getDcPl,dto.getDcPl()).ge(DmYdjh::getZt,32);

        String pgrq = DateUtil.date().toString("yyyy/MM/dd");
        switch (dto.getZt()){
            case "21":
                wrapper.set(DmYdjh::getPwpgrq,pgrq).set(DmYdjh::getPwpgry,dto.getForemanId());
                wrapper.lt(DmYdjh::getZt,31);//仅处理未派工的数据
                break;
            case "42":
            case "52":
                if ("HXPG".equals(dto.getZxZtName())){
                    wrapper.set(DmYdjh::getHxpgrq,pgrq).set(DmYdjh::getHxpgry,dto.getForemanId()).set(DmYdjh::getZt,"51");
                    wrapper.lt(DmYdjh::getZt,51);//仅处理未派工的数据
                }
                if ("QGPG".equals(dto.getZxZtName())){
                    wrapper.set(DmYdjh::getQgpgrq,pgrq).set(DmYdjh::getQgpgry,dto.getForemanId()).set(DmYdjh::getZt,"61");
                    wrapper.lt(DmYdjh::getZt,61);//仅处理未派工的数据
                }
                break;
        }

        this.update(wrapper);
    }

    public DmBzry getForemanByWorker(String worker){
        DmBzry one = bzryService.getOne(Wrappers.<DmBzry>lambdaQuery().eq(DmBzry::getUserCode, worker));
        if ("1".equals(one.getForeman())){
            return one;
        }
        return bzryService.getOne(Wrappers.<DmBzry>lambdaQuery().eq(DmBzry::getUserCode, one.getForeman()));
    }

    public List<DmBzry> getForeman(String zyq){
        return bzryService.list(Wrappers.<DmBzry>lambdaQuery()
                .eq(DmBzry::getForeman, "1").eq(DmBzry::getZyq, zyq));
    }
    /**
     * 获取所有未处理批次
     */
    public List<Map<String,Object>> getWcl(){
        return baseMapper.getWcl();
    }

    public List<Map<String, Object>> gcjd(){
        ImmutableMap<String, Boolean> map = ImmutableMap.of("flag", true);
        List<DmCbxxp> list = dmCbxxpService.getList(map);
        List<String> czbhList=list.stream().map(DmCbxxp::getCbmc).collect(Collectors.toList());
        return baseMapper.gcjd(czbhList);
    }

    public List<Map<String, Object>> gxjhwcqk(){
        DateTime date = DateUtil.date();
        String from= DateUtil.beginOfMonth(date).toString("yyyy/MM/dd");
        String to =DateUtil.endOfMonth(date).toString("yyyy/MM/dd");
        return baseMapper.gxjhwcqk(from,to);
    }

    public List<Map<String, Object>> gyfcl(){
        return baseMapper.gyfcl();
    }

    public List<Map<String, Object>> jhwcqk(){
        DateTime date = DateUtil.date();
        String from= DateUtil.beginOfMonth(date).toString("yyyy/MM/dd");
        String to =DateUtil.endOfMonth(date).toString("yyyy/MM/dd");
        return baseMapper.jhwcqk(from,to);
    }

    public Map<String,Object> gekuachaliang(){
        DateTime date = DateUtil.date();
        String from= DateUtil.beginOfMonth(date).toString("yyyy/MM/dd");
        String to =DateUtil.endOfMonth(date).toString("yyyy/MM/dd");
        return baseMapper.gekuachaliang(from,to,"11");
    }

    public void updateForSl(ShangLiao sl){
        String date=DateUtil.date().toString("yyyy/MM/dd");
        LambdaUpdateWrapper<DmYdjh> wrapper = Wrappers.<DmYdjh>lambdaUpdate()
                //上料
                .set(ObjUtil.isNotEmpty(sl.getYdjh().getSlycldw()),DmYdjh::getSlycldw, sl.getYdjh().getSlycldw())
                .set(ObjUtil.isNotEmpty(sl.getYdjh().getSlyclcs()),DmYdjh::getSlyclcs, sl.getYdjh().getSlyclcs())
                .set(ObjUtil.isNotEmpty(sl.getYdjh().getSlfkry()),DmYdjh::getSlfkrq, date)
                .set(ObjUtil.isNotEmpty(sl.getYdjh().getSlfkry()),DmYdjh::getSlfkry, sl.getYdjh().getSlfkry())

                //理料
                .set(ObjUtil.isNotEmpty(sl.getYdjh().getLldw()),DmYdjh::getLldw, sl.getYdjh().getLldw())
                .set(ObjUtil.isNotEmpty(sl.getYdjh().getLlcs()),DmYdjh::getLlcs, sl.getYdjh().getLlcs())
                .set(ObjUtil.isNotEmpty(sl.getYdjh().getLlfkry()),DmYdjh::getLlfkrq, date)
                .set(ObjUtil.isNotEmpty(sl.getYdjh().getLlfkry()),DmYdjh::getLlfkry, sl.getYdjh().getLlfkry())
                .eq(DmYdjh::getId, sl.getYdjh().getId());

        switch (sl.getType()){
            case YCL:
                wrapper.set(DmYdjh::getZt, DmYdjh.getSlZt());
                break;
            case LLJ:
                wrapper.set(DmYdjh::getZt, DmYdjh.getLLZt());
        }


        this.update(wrapper);
    }

    private static final String KEY="gxControl";

    public Object cacheable(String gx){
        if (gx!=null){
            redisTemplate.opsForValue().set(KEY,gx);
            return gx;
        }else{
            return redisTemplate.opsForValue().get(KEY);
        }
    }
}
